$title  A Static General Equilibrium Analysis

$setglobal dataset aspen_small


SCALAR	RSK	Flag for region-specific capital /1/,
	GK	Flag for global capital		 /0/;

*	Read the dataset:

$batinclude ..\inclib\mrtdata


*	======================================================================
*	Set definitions specific to the EMF models:

set     nr(i,r)         Non-exhaustible energy production
        ele(i)          Electricity				/ele/,
        xe(i)           Exhaustible energy			/cru,gas,col/
        cru(i)          Crude oil				/cru/,
	oil(i)		Refined oil				/oil/,
        gas(i)          Natural gas				/gas/,
        coal(i)         Coal					/col/,
	fe(i)           Final energy				/oil, gas, col/
	e(i)		Energy markets
	     /  gas     Natural gas works
                ele     Electricity and heat 
                oil     Refined oil products
                col     Coal transformation
                cru     Crude oil /;

nr(i,r) = yes$(vom(i,r)$(not xe(i)));

*	======================================================================
*	Scalars and other parameter definitions:

*	Define scalars characterizing the baseline growth
*	along a uniform steady-state path:

*	======================================================================
*	Additional parameters used for model specification:

parameter       
	cecphys(*)      carbon emission coefficients for physical units 
                                /col 0.024, gas 0.0137, oil 0.0181 /

        a0(i,r)         Armington supply
        m0(i,r)         Aggregate import supply
        d0(i,r)         Aggregate domestic supply

        c0(i,r)         Final demand by commodity
        tc(i,r)         Tax rate on final demand
        ct0(r)          Total final demand
        pc0(i,r)        Reference price for final demand

        ld0(i,r)        Base year labor demand by sector
        kd0(i,r)        Base year capital earnings by sector
        
        pmx0(i,s,r)     Reference price of imports
        pmt0(i,s,r)     Reference price of transport services

        rd0(i,r)        Resource inputs for fossil fuel production

        carbcoef(j,*,r) Carbon coefficients
        carblim(r)      Carbon emission limit

        esub_es(i,r)    Elasticity of substitution in energy production
        thetar(i,r)     Value share of resource inputs in fossil supply

        eta(xe)         Elasticity of supply 

	carbon95(r,*)	1995 carbon emissions (IEA)

        ccoef(r,i)	Carbon coefficients
        vshare(r,i)	Energy value shares for non-energy sectors
        carbinv(*,*)	CO2 inventories (IEA)-- mton
        carbint(r,i)	Sectoral CO2 intensities --kg per $output 

	ffs		Fossil fuel production
	energy		Energy use

	pfe0(i,i,r)	Baseline reference price of energy

	energyinp	Energy sector inputs (echo print)

	gr0(r)		Benchmark growth rates
	i0ratio(r)	Ratio of benchmark investment to capital;

*	======================================================================
*	Adopt some notation which is different from the core GTAPinGAMS model:

a0(i,r) = vipm(i,r) + vdpm(i,r) + vigm(i,r) + vdgm(i,r) + vifm(i,r) + vdfm(i,r);
d0(i,r) = vdpm(i,r) + vdgm(i,r) + vdfm(i,r);
m0(i,r) = vipm(i,r) + vigm(i,r) + vifm(i,r);

c0(i,r) = vpm(i,r) + vgm(i,r);
tc(i,r)$c0(i,r) = (tp(i,r)*vpm(i,r) + tg(i,r)*vgm(i,r)) / c0(i,r);
ct0(r) = sum(i, c0(i,r)*(1+tc(i,r)));
pc0(i,r) = 1 + tc(i,r);
ct0(r) = sum(i, pc0(i,r)*c0(i,r));
vg(r) = 0;

ld0(i,r) = vfm("lab",i,r);
kd0(i,r) = vfm("cap",i,r);
evoa("lab",r) = sum(i, ld0(i,r));
evoa("cap",r) = sum(i, kd0(i,r));
        
pmx0(i,s,r) = (1 + tm(i,s,r))*(1+tx(i,s,r));
pmt0(i,s,r) = 1 + tm(i,s,r);

*	Elasticities of fossil fuel supply:

eta("cru")  = 1;
eta("gas")  = 1;
eta("col")  = 1;

*	======================================================================
*       Establish resource inputs for resource supply:

rd0(xe,r) = kd0(xe,r);
kd0(xe,r) = 0;


thetar(xe,r)$VOM(xe,R) = rd0(xe,r) / (vom(xe,r) * (1-ty(xe,r)));
esub_es(xe,R)$(1-thetar(xe,r)) = thetar(xe,r) * ETA(xe) / (1-thetar(xe,r));
display  thetar, esub_es;



*	======================================================================
*	Generate base year carbon emissions by sector and then echo-print some
*	benchmark statistics:

carbcoef(fe,"final",r) = cecphys(fe) * efd(fe,r);
carbcoef(fe,i,r)       = cecphys(fe) * eind(fe,i,r);
carbon95(r,fe) = carbcoef(fe,"final",r) + sum(i, carbcoef(fe,i,r));
carbon95(r,"total") = sum(fe, carbon95(r,fe));

parameter carbshare;
carbshare(r,"final") = sum(fe, carbcoef(fe,"final",r)) / carbon95(r,"total");
carbshare(r,i) = sum(fe, carbcoef(fe,i,r)) / carbon95(r,"total");
display carbshare;


*	Generate a report of energy use in physical units:

energy(e,r,i)       = round(eind(e,i,r), 1);
energy(e,r,"final") = round(efd(e,r), 1);
option energy:1:1:1;
display energy;

*	Energy value shares in non-energy production:

vshare(r,i)$(vom(i,r) and (not e(i))) 
        = round( 100 * sum(e, vafm(e,i,r)*(1+ti(e,i,r))) / vom(i,r), 1);

ccoef(r,fe) = cecphys(fe);

carbinv(r,"ind_nele")=1000 * sum((e,i)$(not ele(i)), ccoef(r,e)*eind(e,i,r));
carbinv(r,"fd_nele")= 1000 * sum(e, ccoef(r,e)*efd(e,r));

ccoef(r,ele) = sum(fe, cecphys(fe) * eind(fe,ele,r))/
                        (sum(i, eind(ele,i,r)) + efd(ele,r));

*	Carbon inventories:

carbinv(r,"electric")=1000*sum((e,ele), ccoef(r,e)*eind(e,ele,r));
carbinv(r,"total")=1000*sum(e, ccoef(r,e)*(sum(i$(not ele(i)), eind(e,i,r))+efd(e,r)) );
carbinv(r,"ind_total")=1000 * sum((e,i)$(not ele(i)), ccoef(r,e)*eind(e,i,r));
carbinv(r,"fd_total")= 1000 * sum(e, ccoef(r,e)*efd(e,r));
carbinv(r,"kg/$")= round(carbinv(r,"total")/(10*sum((f,i), vfm(f,i,r))), 1);

set jrep /total,ind_total, fd_total, ind_nele, fd_nele, electric/;
carbinv("total",jrep) = sum(r, carbinv(r,jrep));

*	Generate reports for carbon intensities in electricity
*	and non-energy sectors:

carbint(r,ele)$vom(ele,r) = 
  round(100*sum(fe, cecphys(fe) * eind(fe,ele,r))/vom(ele,r),1);
carbint(r,i)$(vom(i,r) and (not e(i))) = 
  round(100*sum(e, ccoef(r,e)*eind(e,i,r))/vom(i,r), 1);

option vshare:1, carbinv:1, carbint:1;
display vshare, carbinv, carbint;

energyinp(e,j,r)$vom(e,r)   = vafm(j,e,r)/vom(e,r);
energyinp(e,"l",r)$vom(e,r) = ld0(e,r)/vom(e,r);
energyinp(e,"k",r)$vom(e,r) = kd0(e,r)/vom(e,r);
energyinp(e,"r",r)$vom(e,r) = rd0(e,r)/vom(e,r);
display energyinp;


$ontext

$model:gtap

$sectors:
        c(r)			! Private consumption
        y(i,r)$vom(i,r)		! Output
        a(i,r)$a0(i,r)		! Armington aggregation of domestic and imports
	yt			! Transport 

$commodities:
        pc(r)                   ! Final demand
        py(i,r)$vom(i,r)	! Output price price
        pa(i,r)$a0(i,r)		! Armington composite price
        pl(r)                   ! Wage rate
        pr(i,r)$rd0(i,r)        ! Energy resource
	rkr(r)$rsk		! Return to regional capital
	rkg$gk			! Return to global capital
        pt                      ! Transport services
        pcarb(r)		! Carbon permits -- non-tradable 

$consumers:
        ra(r)                   ! Representative agent

*       Final demand



$prod:c(r)  s:0.5 c:1 e:1  oil(e):0 col(e):0 gas(e):0
    o:pc(r)	q:ct0(r)
    i:pa(i,r)	q:c0(i,r)   p:pc0(i,r) i.tl:$fe(i) c:$(not e(i)) e:$ele(i) a:ra(r) t:tc(i,r) 
    i:pcarb(r)#(fe)	q:carbcoef(fe,"final",r) p:1e-6  fe.tl:

$report:
        v:edf(i,r)$(e(i) and c0(i,r)) i:pa(i,r) prod:c(r)	! energy in final demand
	v:fd(xe,r)                    o:py(xe,r) prod:y(xe,r)   ! fossil fuel supply

*       Non-fossil fuel production (includes electricity and refining):

$prod:y(i,r)$nr(i,r)   s:0  vae(s):0.5  va(vae):1
+                       e(vae):0.1  nel(e):0.5 lqd(nel):2
+                       oil(lqd):0 col(nel):0 gas(lqd):0 

	o:py(i,r)	q:vom(i,r)  a:ra(r) t:ty(i,r) 
	i:pa(j,r)$(not fe(j))	q:vafm(j,i,r) p:pai0(j,i,r) e:$ele(j) a:ra(r) t:ti(j,i,r)
        i:pl(r)		q:ld0(i,r)                      va:
        i:rkr(r)$rsk	q:kd0(i,r)                      va:
        i:rkg$gk	q:kd0(i,r)                      va:
        i:pcarb(r)#(fe)	q:carbcoef(fe,i,r) p:1e-6   fe.tl:
        i:pa(fe,r)	q:vafm(fe,i,r)	p:pai0(fe,i,r) fe.tl: a:ra(r) t:ti(fe,i,r) 

$report:
        v:ed(j,i,r)$(fe(j) and vafm(j,i,r)) i:pa(j,r) prod:y(i,r)

*       Fossil fuel production activity (crude, gas and coal):

$prod:y(xe,r)$vom(xe,r)  s:(esub_es(xe,r))  id:0 
        o:py(xe,r)	q:vom(xe,r)	a:ra(r) t:ty(xe,r) 
        i:pa(j,r)	q:vafm(j,xe,r)	p:pai0(j,xe,r)  a:ra(r) t:ti(j,xe,r) id:
        i:pl(r)		q:ld0(xe,r)  id:
        i:pr(xe,r)	q:rd0(xe,r)	

*       Armington aggregation over domestic versus imports:

$prod:a(i,r)$a0(i,r)  s:4  m:8 s.tl(m):0
        o:pa(i,r)       q:a0(i,r)
        i:py(i,r)       q:d0(i,r)
        i:py(i,s)       q:vxmd(i,s,r)  p:pmx0(i,s,r) s.tl:
+               a:ra(s) t:tx(i,s,r)     a:ra(r) t:(tm(i,s,r)*(1+tx(i,s,r)))
        i:pt#(s)       q:vtwr(i,s,r)  p:pmt0(i,s,r) s.tl: a:ra(r) t:tm(i,s,r)

*       International transport services (Cobb-Douglas):

$prod:yt  s:1
        o:pt           q:(sum((i,r), vst(i,r)))
        i:py(i,r)       q:vst(i,r)

*       Final demand:

$demand:ra(r)   
        d:pc(r)		q:ct0(r)
	e:py("cgd",r)	q:-vom("cgd",r)
	e:rkr(r)$rsk	q:(sum(i, kd0(i,r)))
	e:rkg$gk	q:(sum(i, kd0(i,r)))
        e:pl(r)		q:evoa("lab",r)
        e:pr(xe,r)	q:rd0(xe,r)
        e:pc("usa")	q:vb(r)
        e:pcarb(r)	q:carblim(r)

$report:
	v:w(r)	w:ra(r)

$offtext
$sysinclude mpsgeset gtap

*	No carbon taxes in the benchmark:

pcarb.l(r) = 0;

carblim(r) = 500000;

carbcoef(fe,"final",r)$(not c0(fe,r)) = 0;
carbcoef(fe,i,r)$(not vafm(fe,i,r)) = 0;


carblim(r)$annexb(r) = sum(fe, carbcoef(fe,"final",r)+sum(i,carbcoef(fe,i,r)));


gtap.iterlim = 0;
$include gtap.gen
solve gtap using mcp;
display "The following reports precision of the benchmark dataset:", gtap.objval;

parameter baucarbon, scncarbon;

baucarbon(r) = sum(fe, cecphys(fe) * (
	 (  (edf.l(fe,r)/c0(fe,r)) *  efd(fe,r))$c0(fe,r) +
         sum(i$vafm(fe,i,r), (ed.l(fe,i,r)/vafm(fe,i,r)) * eind(fe,i,r)) ));
baucarbon("total") = sum(r,baucarbon(r));

display baucarbon;

gtap.iterlim = 8000;
carblim(r)$annexb(r) = 0.75 * carblim(r);
$include gtap.gen
solve gtap using mcp;


scncarbon(r) = sum(fe, cecphys(fe) * (
	 (  (edf.l(fe,r)/c0(fe,r)) *  efd(fe,r))$c0(fe,r) +
         sum(i$vafm(fe,i,r), (ed.l(fe,i,r)/vafm(fe,i,r)) * eind(fe,i,r)) ));
scncarbon("total") = sum(r,scncarbon(r));


display scncarbon;

parameter leakage, totleakage, regleakage;

leakage(r)$(not annexb(r)) = 100 * (scncarbon(r) - baucarbon(r)) /
			     sum(annexb(rr), baucarbon(rr) - scncarbon(rr));

leakage("total")=sum(r, leakage(r));

totleakage = leakage("total");
regleakage(r) = leakage(r);

display totleakage, regleakage;
